PROGRAM IterationFunctionSystem;

USES
  Graph;

CONST
  MAXDATA = 4;
  INF     = 1E35;

TYPE
  IFS_data = Record
               a : Array [ 1..MAXDATA ] of Real;
               b : Array [ 1..MAXDATA ] of Real;
               c : Array [ 1..MAXDATA ] of Real;
               d : Array [ 1..MAXDATA ] of Real;
               e : Array [ 1..MAXDATA ] of Real;
               f : Array [ 1..MAXDATA ] of Real;
               Prob : Array [ 1..MAXDATA ] of Integer;
             End;

PROCEDURE IFS( Var Data : IFS_data;
                   Iteration : LongInt;
                   Xscale, Yscale, Xoffset, Yoffset : Integer );
Var
  x,y,newx,newy : Real;
  Prob          : Integer;
  I             : LongInt;
  K             : Integer;

Begin
  x := 0.0;
  y := 0.0;
  For I := 1 to Iteration do
    begin
      Prob := Random( 100 );
      If Prob <= Data.Prob[1] Then
        K := 1
      Else if Prob <= Data.Prob[2] Then
        K := 2
      Else if Prob <= Data.Prob[3] Then
        K := 3
      Else
        K := 4;

      With Data do
        begin
          newx := a[k]*x + b[k]*y + e[k];
          newy := c[k]*x + d[k]*y + f[k];
        end;
      x := newx;
      y := newy;

      If ( x > INF ) or ( y > INF ) Then Exit;
      If I > 10 Then
        PutPixel( Trunc( x*xscale ) + xoffset,
                  Trunc( y*yscale ) + yoffset, 1 );
    end;
End;

Var
  DragonSweep,
  Fractal_Tree,
  Fern,
  Sierpinski    : IFS_data;
  Gd,Gm         : Integer;

BEGIN
  Randomize;
  With DragonSweep do
    begin
      a[1] := 0.5; a[2] := 0.5;
      b[1] :=-0.5; b[2] :=-0.5;
      c[1] := 0.5; c[2] := 0.5;
      d[1] := 0.5; d[2] := 0.5;
      e[1] := 1.0; e[2] :=-1.0;
      f[1] := 0;   f[2] := 0;
      Prob[1] := 50; Prob[2] :=100;
    end;

  With Sierpinski do
    begin
      a[1] := 0.5; a[2] := 0.5; a[3] := 0.5;
      b[1] := 0.0; b[2] := 0.0; b[3] := 0.0;
      c[1] := 0.0; c[2] := 0.0; c[3] := 0.0;
      d[1] := 0.5; d[2] := 0.5; d[3] := 0.5;
      e[1] := 0.0; e[2] := 1.0; e[3] := 0.5;
      f[1] := 0.0; f[2] := 0.0; f[3] := 0.5;
      Prob[1] := 33;
      Prob[2] := 66;
      Prob[3] := 100;
    end;

  With Fern do
    begin
      a[1] := 0.0;  a[2] := 0.2;   a[3] := -0.15; a[4] := 0.85;
      b[1] := 0.0;  b[2] := -0.26; b[3] := 0.28;  b[4] := 0.04;
      c[1] := 0.0;  c[2] := 0.23;  c[3] := 0.26;  c[4] := -0.04;
      d[1] := 0.16; d[2] := 0.22;  d[3] := 0.24;  d[4] := 0.85;
      e[1] := 0.0;  e[2] := 0.0;   e[3] := 0.0;   e[4] := 0.0;
      f[1] := 0.0;  f[2] := 1.6;   f[3] := 0.44;  f[4] := 1.6;
      Prob[1] := 1; Prob[2] := 8; Prob[3] := 15; Prob[4] := 100;
    end;

  With Fractal_Tree do
    begin
      a[1] := 0.0;  a[2] := 0.1;   a[3] :=  0.42; a[4] := 0.42;
      b[1] := 0.0;  b[2] := 0.0;   b[3] :=-0.42;  b[4] := 0.42;
      c[1] := 0.0;  c[2] := 0.0;   c[3] := 0.42;  c[4] := -0.42;
      d[1] := 0.5;  d[2] := 0.1;   d[3] := 0.42;  d[4] := 0.42;
      e[1] := 0.0;  e[2] := 0.0;   e[3] := 0.0;   e[4] := 0.0;
      f[1] := 0.0;  f[2] := 0.2;   f[3] := 0.2;   f[4] := 0.2;
      Prob[1] := 5; Prob[2] :=20; Prob[3] := 60; Prob[4] := 100;
    end;

  Gd := Detect;
  InitGraph( Gd,Gm, '' );

  IFS( DragonSweep, 50000,100,70,350,170 );
  OutTextXY( 500,10, 'Press RETURN' );
  Readln;

  ClearDevice;
  IFS( Sierpinski,10000,300,280,50,50 );
  OutTextXY( 500,10, 'Press RETURN' );
  Readln;

  ClearDevice;
  IFS( Fractal_Tree,10000,800,700,200,0 );
  OutTextXY( 500,10, 'Press RETURN' );
  Readln;

  ClearDevice;
  IFS( Fern,50000,35,32,300,10 );
  Readln;

  CloseGraph;

END.
