PROGRAM MANDELBROT_SET;

USES
  Crt,Graph;

CONST
  MaxCount = 255;
  UpLtX = 20;
  UpLtY = 20;

VAR
  Gd,Gm,AXsize,AYsize,
  X,Y,Count,
  CurX,CurY,CurSize    : Integer;
  Xmin,Xmax,Ymin,Ymax,
  Za,Zb,Ca,Cb,b1,
  a,b,dX,dY,Size       : Real;
  Stop                 : Boolean;

PROCEDURE PutTitle;
  Var
    Dum1,Dum2 : String[10];

  Begin
    ClearViewPort;

    OutTextXY( GetMaxX - 250,0, '** Mandelbrot Set **' );
    SetColor( White );
    Rectangle( UpLtX,UpLtY, UpLtX+AXsize+1,UpLtY+AYsize+1 );

    Str( Xmin:5,Dum1 );
    Str( Xmax:5,Dum2 );
    OutTextXY( GetMaxX - 100,20,'X :' + Dum1 );
    OutTextXY( GetMaxX - 100,40,' ->' + Dum2 );
    Str( Ymin:5,Dum1 );
    Str( Ymax:5,Dum2 );
    OutTextXY( GetMaxX - 100,70,'Y :' + Dum1 );
    OutTextXY( GetMaxX - 100,90,' ->' + Dum2 );

  End;

PROCEDURE GetNewBd;
  Var
    CurX,CurY,CurSize    : Integer;
    Ch                   : Char;

  Begin
    CurX := UpLtX+1; CurY := UpLtY+1; CurSize := 50;

    SetColor( White );
    SetWriteMode( XORput );
    Rectangle( CurX,CurY,
               CurX+CurSize, CurY+CurSize );

    Repeat
      Ch := ReadKey;
      Rectangle( CurX,CurY,
                 CurX+CurSize, CurY+CurSize );
      If Ch = #0 Then
        Case ReadKey of
          #72 : CurY := CurY - 10;
          #75 : CurX := CurX - 10;
          #77 : CurX := CurX + 10;
          #80 : CurY := CurY + 10;
          #15 : If CurSize > 5 Then CurSize := CurSize - 10;
        End
      Else If Ch = #9 Then
        CurSize := CurSize + 10
      Else If Ch = #27 Then
        Stop := TRUE;

      Rectangle( CurX,CurY,
                 CurX+CurSize, CurY+CurSize );
    Until Ch = #13;

    Xmin := Xmin + (CurX-UpLtX-1)*dX;
    Xmax := Xmin + CurSize*dX;
    Ymin := Ymin + (CurY-UpLtY-1)*dY;
    Ymax := Ymin + CurSize*dY;

  End;

BEGIN

  Write ( 'X : from: ' ); Readln( Xmin );
  Write ( '    to  : ' ); Readln( Xmax );
  Write ( 'Y : from: ' ); Readln( Ymin );
  Ymax := Ymin + Xmax - Xmin;

  Writeln;
  Write ( 'Size : ' ); Readln( AXsize );
  AYsize := AXsize;

  Gd := Detect;
  InitGraph ( Gd,Gm,'' );

  Stop := FALSE;

  Repeat

    PutTitle;

    dX := ( Xmax - Xmin ) / AXsize;
    dY := ( Ymax - Ymin ) / AYsize;

    Za := Xmin;
    Zb := Ymin;

      For X := 0 to 4 do
        Line( UpLtX + X*AXsize div 4, UpLtY - 10,
              UpLtX + X*AXsize div 4, UpLtY       );
      For Y := 0 to 4 do
        Line( UpLtX - 10, UpLtY + Y*AYsize div 4,
              UpLtX,      UpLtY + Y*AYsize div 4  );

    For Y := 1 to AYsize do
      begin
        For X := 1 to AXsize do
          begin
            Count := 0;
            a := Za;
            b := Zb;
            Repeat
              b1 := 2*a*b;
              a := a*a - b*b + Za;
              b := b1 + Zb;
              Size := a*a + b*b;
              Count := Count + 1;
            Until ( Size > 4.0 ) or ( Count > MaxCount );
            PutPixel ( UpLtX + X, UpLtY + Y, Count mod (GetMaxColor+1) );
            Za := Za + dX;
          end;
        Zb := Zb + dY;
        Za := Xmin;
        If KeyPressed Then Y := Aysize;
      end;

    GetNewBd;

  Until Stop;

END.

